from .models import User
from rest_framework import serializers
from rest_framework_jwt.settings import api_settings
from rest_framework.exceptions import ValidationError, APIException
from django.db.models import Q
from django.conf import settings
from django.core.cache import cache
import re
from faker import Faker
jwt_payload_handler = api_settings.JWT_PAYLOAD_HANDLER
jwt_encode_handler = api_settings.JWT_ENCODE_HANDLER


class CommonLoginSerializer:
    def _get_user(self, attrs):
        raise APIException('你必须重写')

    def _get_token(self, user):
        try:
            payload = jwt_payload_handler(user)
            token = jwt_encode_handler(payload)
            return token
        except Exception as e:
            raise ValidationError(str(e))

    def validate(self, attrs):
        # 校验用户是否存在
        user = self._get_user(attrs)
        # 签发token
        token = self._get_token(user)
        # 把用户名、token、头像放在上下文中【序列化类的context】
        self.context['username'] = user.username
        self.context['token'] = token
        self.context['icon'] = settings.BACKEND_URL + user.icon.url
        return attrs  # 接下来传到update或者ser.save中，这里没有可不写


class UserMulLoginSerializer(CommonLoginSerializer, serializers.ModelSerializer):
    username = serializers.CharField(required=True)

    class Meta:
        model = User
        fields = ['username', 'password']

    def _get_user(self, attrs):
        username = attrs.get('username')
        password = attrs.get('password')
        user = User.objects.filter(Q(username=username) | Q(mobile=username) | Q(email=username)).first()
        if user and user.check_password(password):
            return user
        raise ValidationError('用户名或密码错误')


class UserSmsLoginSerializer(CommonLoginSerializer, serializers.Serializer):
    mobile = serializers.CharField(required=True)
    code = serializers.CharField(max_length=4)

    def _get_user(self, attrs):
        mobile = attrs.get('mobile')
        code = attrs.get('code')
        old_code = cache.get('send_sms_code_%s' % mobile)
        # 验证验证码是否正确
        if code == old_code or (settings.DEBUG and code == '6666'):
            user = User.objects.filter(mobile=mobile).first()
            if user:
                return user
            else:
                raise APIException('手机号没注册')
        else:
            raise APIException('验证码错误')


class UserRegisterSerializer(CommonLoginSerializer, serializers.Serializer):
    mobile = serializers.CharField(required=True)
    password = serializers.CharField(required=True)
    code = serializers.CharField(max_length=4, min_length=4)

    def validate_mobile(self, mobile):
        pattern = r'^(13[0-9]|14[5-9]|15[0-3,5-9]|16[2,5,6,7]|17[0-9]|18[0-9]|19[1,8,9])\d{8}$'
        if not re.match(pattern, mobile):
            raise ValidationError('手机号码不符合国内规范')
        return mobile

    def _check_code(self, attrs):
        code = attrs.get('code')
        mobile = attrs.get('mobile')
        old_code = cache.get('send_sms_code%s' % mobile)
        if not (code == old_code or (settings.DEBUG and code == '6666')):
            raise ValidationError('验证码错误')

    def _pre_save(self, attrs):
        fake = Faker('zh_CN')
        attrs.pop('code')
        attrs['username'] = fake.name()

    def validate(self, attrs):
        self._check_code(attrs)
        self._pre_save(attrs)
        return attrs

    def create(self, validated_data):
        # 密码加密或者在钩子里做make_password
        user = User.objects.create_user(**validated_data)
        return user
